/* src/add_info.c - Ajout d'informations en mmoire
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: add_info.c,v 1.57 2003/12/28 01:07:32 romexzf Exp $
 */

#include "main.h"
#include "debug.h"
#include "outils.h"
#include "hash.h"
#include "ban.h"
#include "cs_cmds.h"

#ifdef USE_MEMOSERV
void add_memo(anUser *nick, const char *de, time_t date, const char *message, int read)
{
	aMemo *new = calloc(1, sizeof(aMemo)), *current = nick->memohead;

	if(!new)
	{
		Debug(W_MAX, "add_memo, malloc a chou pour aMemo %s (%s)", nick->nick, message);
		return;
	}

	new->next = NULL;
	Strncpy(new->de, de, NICKLEN);
	new->date = date;
	new->read = read;
	Strncpy(new->message, message, MEMOLEN);

	if(!nick->memohead) nick->memohead = new;
	else
	{
		for(;current->next;current = current->next);
		current->next = new;
	}
}
#endif

struct badchans *add_abadchan(const char *chaine, const char *raison)
{
	struct badchans *new = calloc(1, sizeof(struct badchans));

	if(!new)
	{
		Debug(W_MAX, "add_abadchan, malloc a chou pour le badchan %s", chaine);
		return NULL;
	}

	Strncpy(new->chaine, chaine, 99);
	Strncpy(new->raison, raison, 149);
	new->next = badchead;
	badchead = new;
	return new;
}

anAccess *add_access(anUser *user, const char *chan, int level, int flag, time_t lastseen)
{
	anAccess *new = calloc(1, sizeof(anAccess)), *current;
	aChan *c;
	aLink *lp;

	if(!new)
	{
		Debug(W_MAX, "add_access, malloc a chou pour l'accs %s (%s)", user->nick, chan);
		return NULL;
	}
	if(!user || !chan)
	{
		Debug(W_MAX, "add_access appel avec trop peu d'argument user=%s chan%s", user ? user->nick : "", NONE(chan));
		free(new);
		return NULL;
	}
	new->next = NULL;
	new->level = level;
	new->flag = flag;
	new->user = user;
	new->info = NULL;
	new->lastseen = lastseen ? lastseen : user->lastseend;

	if((c = getchaninfo(chan))) new->c = c;
	else
	{
		Debug(W_DESYNCH, "add_access, accs de %s sur %s, non reg ?", user->nick, chan);
		free(new);
		return NULL;
	}

	if(!user->accesshead) user->accesshead = new;
	else
	{
		for(current = user->accesshead;current->next && current->c != c;current = current->next);
		if(!current->next) current->next = new;
		else
		{
			Debug(W_DESYNCH, "add_access, accs de %s sur %s dj prsent?", user->nick, chan);
			free(new);
			return NULL;
		}
	}

	lp = calloc(1, sizeof(aLink));
	lp->value.a = new;
	lp->next = c->access;
	c->access = lp;
	return new;
}

void add_killinfo(aNick *nick, time_t timestamp, int why, int setkill)
{
	aKill *new = calloc(1, sizeof(aKill)), *current;

	if(!new)
	{
		Debug(W_MAX|W_WARN, "add_killinfo, malloc a chou pour aKill %s", nick->nick);
		return;
	}

	new->next = NULL;
	new->nick = nick;
	Strncpy(new->against, nick->nick, NICKLEN);
	new->timestamp = timestamp;
	new->why = why;
	new->setkill = setkill;
	if(!killhead) killhead = new;
	else
	{
		for(current = killhead;current->next;current = current->next);
		current->next = new;
	}
}

int add_abadnick(const char *chaine, int flag)
{
	struct badnicks *new = calloc(1, sizeof(struct badnicks));

	if(!new)
		return Debug(W_MAX|W_WARN, "add_abadnick, malloc a chou pour le badnick %s", chaine);

	Strncpy(new->chaine, chaine, NICKLEN);
	new->flag = flag;
	new->next = badnhead;
	badnhead = new;
	return 1;
}

void add_join(aNick *nick, char *chan, int status, time_t timestamp, aChan *chaninfo)
{

	aJoin *new = calloc(1, sizeof(aJoin)), *join;
	anAccess *access;
	aLink *lp;

	if(!new)
	{
		Debug(W_MAX|W_WARN, "add_join, malloc a chou pour aJoin de %s sur %s", nick->nick, chan);
		return;
	}

	new->next = NULL;
	Strncpy(new->channel, chan, CHANLEN);
	new->status = (status & ~J_BURST);
	new->timestmp = timestamp;
	new->nick = nick;
	new->chan = NULL;
	if(!nick->joinhead) nick->joinhead = new;
	else
	{
		for(join = nick->joinhead;join->next && strcasecmp(join->channel, chan);join = join->next);
		if(!join->next && strcasecmp(join->channel, chan)) join->next = new;
		else /* le mec est dja sur le chan ? */
		{
			free(new);
			Debug(W_DESYNCH|W_WARN, "reJOIN %s de %s", nick->nick, chan);
			new = join;
		}
	}

	if((!(status & J_BURST) && !(chaninfo = getchaninfo(chan))) || !chaninfo || IsSuspend(chaninfo)) return;

	new->chan = chaninfo;
	chaninfo->users++;

	/* on ajoute le link vers chanmembers uniquement si le salon n'est PAS suspendu
	 pb vu lors du burst d'un chan non suspendu */
	lp = calloc(1, sizeof(aLink));
	lp->value.j = new;

	lp->next = chaninfo->members; /* insertion du link dans les chan members*/
	chaninfo->members = lp;

	if(status & J_BURST) return;
	else
	{ /*ban check (mis ici pr eviter 2 loops)*/
		if(!(nick->flag & (N_OPER | N_SERVICE)) && (!nick->user || !IsAnAdmin(nick->user)))
		{
			aBan *ban = is_ban(nick, chaninfo);
			if(ban)
			{
				csmode(chaninfo, MODE_OBV, "+b %s", ban->mask);
				cskick(chan, nick->numeric, "Enforce: %s", ban->raison);
				return;
			}
		}

		if(CSetWelcome(chaninfo) && *chaninfo->welcome) /*si le welcome est activ ET que le welcome existe*/
			csntc(nick, GetReply(nick, L_WELCOMEJOIN), chaninfo->chan, chaninfo->welcome);
	}
	/*On regarde l'access ici pour infoline, autoop et autovoice*/
	if(!nick->user || IsSuspend(nick->user)) return; /*ne va pas plus loin si l'user n'est pas auth ou qu'il est suspend (peut etre inutile)*/

	if(!(access = GetAccessIbyUserI(nick->user, chaninfo)) || ASuspend(access)) return;/*ne va pas plus loin si l'user n'a pas d'access ou qu'il est suspendu*/
	access->lastseen = 1;

	if(AVoice(access)) DoVoice(new);/* auto op/voice*/
	if(!CNoOps(chaninfo) && AOp(access)) DoOp(new);

	if(new->status) csmode(chaninfo, MODE_OBV, "+%s%s %s %s", IsOp(new) ? "o" : "", IsVoice(new) ? "v" : "",
			IsOp(new) ? nick->numeric : "" , IsVoice(new) ? nick->numeric : "");

	if(access->info) putserv("%s " TOKEN_PRIVMSG " %s :[\2%s\2] %s", cs.num, chan, nick->user->nick, access->info);

	return;
}

void add_ban(aChan *cchan, const char *mask, const char *raison, const char *de, time_t debut, time_t fin, int level)
{
	aBan *new = calloc(1, sizeof(aBan)), *current;

	if(!new)
	{
		Debug(W_MAX|W_WARN, "add_ban, malloc a chou pour aBan %s sur %s de %s (%s)", mask, cchan->chan, de, raison);
		return;
	}

	Strncpy(new->mask, mask, 99);
	str_dup(&new->raison, raison);
	Strncpy(new->de, de, NICKLEN);
	new->debut = debut;
	new->fin = fin;
	new->level = level;
	new->next = NULL;
	new->last = NULL;
	if(!cchan->banhead) cchan->banhead = new;
	else
	{
		int i = 0;
		for(current = cchan->banhead;(i = match(current->mask, mask)) && current->next;current=current->next);
		if(i && !current->next)
		{
			current->next = new;
			new->last = current;
		}
		else free(new);
	}
	return;
}

void add_ignore(aNick *nick, time_t expiration)
{
	struct ignore *new = malloc(sizeof(struct ignore));

	if(!new)
	{
		Debug(W_MAX|W_WARN, "add_ignore, malloc a chou pour l'ignore de %s", nick->host);
		return;
	}

	strcpy(new->host, nick->base64);
	new->exp = expiration;

	new->next = ignorehead;
	ignorehead = new;
	return;
}

