/* src/admin_user.c - commandes admins pour gerer les users
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: admin_user.c,v 1.98 2003/12/27 16:19:25 romexzf Exp $
 */

#include <sys/time.h>
#include "main.h"
#include "outils.h"
#include "hash.h"
#include "cs_cmds.h"
#include "fichiers.h"
#include "divers.h"
#ifdef SQLLOG
#include "sql_log.h"
#include <mysql.h>
#endif
#ifdef HAVE_TRACK
#include "track.h"
#endif
#ifdef HAVE_VOTE
#include "vote.h"
#endif
#ifdef USEBSD
#include <netinet/in.h>
#endif
#include <arpa/inet.h>
#include "admin_user.h"

static char *GetIP(const char *base64)
{
	struct in_addr addr2;

	addr2.s_addr = htonl(base64toint(base64));
	return inet_ntoa(addr2);
}

int cs_whois(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char chans[451] = {0}, b[CHANLEN + 4] = {0}, *ptr;
	int size = 0, first = 0;
	anUser *uptr;
	aJoin *join;
	aNick *nptr = getnickbynick(parv[1]);

	if(!nptr) return csntc(nick, GetReply(nick, L_NOSUCHNICK), parv[1]);

	csntc(nick, "Infos sur \2%s", nptr->nick);
	csntc(nick, " Nom: %s", nptr->name);
#ifdef HAVE_CRYPTHOST
	csntc(nick, " Mask: %s\2%s\2!\002%s\2@\002%s", GetPrefix(nptr), nptr->nick, nptr->ident, nptr->crypt);
	csntc(nick, " Realhost: \002%s\2", nptr->host);
#else
	csntc(nick, " Mask: %s\2%s\2!\002%s\2@\002%s", GetPrefix(nptr), nptr->nick, nptr->ident, nptr->host);
#endif
	csntc(nick, " IP:\2 %s", GetIP(nptr->base64));
	if(IsOper(nptr)) csntc(nick, " %s est \2IRCop.", nptr->nick);
	ptr = GetModes(nptr->flag);
	csntc(nick, " UserModes: \2%s\2", *ptr ? ptr : "Aucun");

	for(join = nptr->joinhead;join; join = join->next)
	{
		int tmps = size, p = sprintf(b, "%s%s%s ", IsVoice(join) ? "+" : "", IsOp(join) ? "@" : "", join->channel);
		size += p;
		if(size >= 450) /* le dernier fait depasser le buffer */
		{
			csntc(nick, !first ? " Prsent sur %s ..." : "%s ...", chans);
			strcpy(chans, b); /* ajoute le suivant pour la prochaine*/
			first = 1;
			size = p;
		}
		else strcpy(&chans[tmps], b);
	}

	csntc(nick, !first ? " Prsent sur %s" : "%s", chans[0] ? chans : "aucun salon.");

	if(nptr->user)
	{
		anAccess *acces = nptr->user->accesshead;
		*chans = 0;
		size = first = 0;

		csntc(nick, " %s est logu sous l'username \2%s\2%s", nptr->nick, nptr->user->nick,
			IsAnAdmin(nptr->user) ? " (\2Admin\2)" : "");

		for(;acces;acces = acces->next)
		{
			int tmps = size, p;
			if(AWait(acces)) continue;

			size += (p = strlen(acces->c->chan));
			if(size >= 450)
			{
				csntc(nick, !first ? GetReply(nick, L_ACCESSON) : "%s ...", chans);
				size = p;
				strcpy(chans, acces->c->chan); /* ajoute le suivant pour la prochaine*/
				first = 1;
			}
			else strcpy(&chans[tmps], acces->c->chan);
			chans[size++] = ' ';
			chans[size] = 0;
		}

		if(*chans) csntc(nick, !first ? GetReply(nick, L_ACCESSON) : "%s", chans);
		else if(!first) csntc(nick, GetReply(nick, L_NOACCESS));
	}
	else if((uptr = getuserinfo(parv[1]))) csntc(nick, " %s est un username enregistr. %s",
				uptr->nick, IsAnAdmin(uptr) ? " (\2Admin\2)" : "");

	csntc(nick, " Connect depuis %s sur \2%s", duration(time(NULL) - nptr->ttmco), nptr->serveur->serv);
	return 1;
}

int admin_user(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *user = parv[2];
	const char *arg = parv[1];
	anUser *u = NULL;
	anAccess *a, *a2;
	aNick *n;

	if(!strcasecmp(arg, "info"))
	{
		time_t ttm = time(NULL);
		int mail = getoption("-mail", parv, parc, 2, 0);/* critres de recherche */
		int seen = getoption("-seen", parv, parc, 2, 0);
		int date = getoption("-date", parv, parc, 2, 1);
		int count = getoption("-count", parv, parc, 2, 1);
		int auth = getoption("-auth", parv, parc, 2, -1);
		int suspend = getoption("-suspend", parv, parc, 2, -1);
		int oubli = getoption("-oubli", parv, parc, 2, -1);
		int first = getoption("-first", parv, parc, 2, -1);
		int nopurge = getoption("-nopurge", parv, parc, 2, -1), cur = 0, nolimit = 0, i = 0;
		time_t seent = (seen > 0) ? convert_duration(parv[seen]) : 0;
		if(seent < 0) seent = 0;

		if(*parv[2] == '-') user = NULL;
		else if(!strchr(user, '?') && !strchr(user, '*'))
		{
			if(!(u = getuserinfo(user))) return csntc(nick, GetReply(nick, L_NOSUCHUSER), user);
			else return show_userinfo(nick, u, 1);
		}
		if(!strcasecmp("-all", parv[parc])) nolimit = 1;

		for(;i < USERHASHSIZE;i++) for(u = user_tab[i];u;u = u->next)
		{
			if((!mail || !match(parv[mail], u->mail))
			&& (!seent || (ttm - u->lastseend) <= seent)
			&& (!date || u->lastseend <= date)
			&& (!auth || (auth && u->n) || (!auth && !u->n))
			&& (!suspend || IsSuspend(u))
			&& (!oubli || UOubli(u))
			&& (!first || UFirst(u))
			&& (!nopurge || UNopurge(u))
			&& (!user || !match(user, u->nick)))
			{
				cur++;
				if(((count && cur > count) || (!count && cur > MAXMATCHES)) && !nolimit) continue;

				show_userinfo(nick, u, 1);
				if(!count || count > 1) csntc(nick, "-");
			}
		}

		if(cur > MAXMATCHES && !nolimit)
			csntc(nick, "[%d/%d] Trop de matches trouves, restreignez votre recherche ou forcez avec '-all' en DERNIER argument.", MAXMATCHES, cur);
		else if(count && count < cur) csntc(nick, "Un Total de %d entre%s trouve%s (%d listes)", cur, PLUR(cur), PLUR(cur), count);
		else if(cur) csntc(nick, "Un Total de %d entre%s trouve%s", cur, PLUR(cur), PLUR(cur));
		else csntc(nick, "Aucun Username correspondant \2 %s\2 trouv.", user ? user : "*");
	}

	else if(!strcasecmp(arg, "match"))
	{
		char *p = parv[2];
		int w = 1, host = 0, nicki = 0, ident = 0, real = 0, join = 0, par = 2;
		int mode = 0, ac = 0, count = 0, crypt = 0, serv = 0, nolimit = 0, i = 0;
		struct timeval tv;
		long ticks;
		aServer *link = NULL;

		if(*p != '+' && *p != '-')
			return csntc(nick, "Syntaxe %s MATCH [+|-]nughCcsam [args..]", RealCmd("user"));

		gettimeofday(&tv, NULL);
		ticks = tv.tv_usec;

		for(;*p;p++)
		{
			char err = 0;
			switch(*p)
			{
				case '+': w = 1; break;
				case '-': w = 0; break;
				case 'h':
						if((host = ++par) <= parc) { if(!w) host = -host; }
						else err = 1;
						break;
				case 'n':
						if((nicki = ++par) <= parc) { if(!w) nicki = -nicki; }
						else err = 1;
						break;
				case 'u':
						if((ident = ++par) <= parc) { if(!w) ident = -ident; }
						else err = 1;
						break;
				case 'g':
						if((real = ++par) <= parc) { if(!w) real = -real; }
						else err = 1;
						break;
				case 'c':
						if((join = ++par) <= parc) { if(!w) join = -join; }
						else err = 1;
						break;
				case 'm':
						if(++par <= parc)
						{
							mode = parse_umode(0, parv[par]);
							if(!w) mode = -mode;
						}
						else err = 1;
						break;
				case 'a':
						if((ac = ++par) <= parc) { if(!w) ac = -ac; }
						else err = 1;
						break;
				case 's':
						if((serv = ++par) <= parc)
						{
							if(!(link = GetLinkIbyServ(parv[serv]))) return csntc(nick, "Serveur %s inconnu.", parv[serv]);
							if(!w) serv = -serv;
						}
						else err = 1;
						break;
#ifdef HAVE_CRYPTHOST
				case 'C':
						if((crypt = ++par) <= parc) { if(!w) crypt = -crypt; }
						else err = 1;
						break;
#endif
				default:
						err = 1;
			}
			if(err) return csntc(nick, "Syntaxe %s MATCH [+|-]nughCcsam [args..]", RealCmd("user"));
		}

		if(!strcasecmp("-all", parv[parc])) nolimit = 1;

		for(;i < NICKHASHSIZE;i++) for(n = nick_tab[i];n;n = n->next)
			if((!nicki || (nicki > 0 && !match(parv[nicki], n->nick)) || (nicki < 0 && match(parv[-nicki], n->nick)))
				&& (!ident || (ident > 0 && !match(parv[ident], n->ident)) || (ident < 0 && match(parv[-ident], n->ident)))
				&& (!real || (real > 0 && !match(parv[real], n->name)) || (real < 0 && match(parv[-real], n->name)))
				&& (!serv || (serv > 0 && link == n->serveur) || (serv < 0 && link != n->serveur))
				&& (!host || (host > 0 && !match(parv[host], n->host)) || (host < 0 && match(parv[-host], n->host)))
				&& (!mode || (mode > 0 && (n->flag & mode) == mode) || (mode < 0 && !(n->flag & -mode)))
				&& (!ac || (n->user && ((ac > 0 && !match(parv[ac], n->user->nick)) || (ac < 0 && match(parv[-ac], n->user->nick)))))
#ifdef HAVE_CRYPTHOST
				&& (!crypt || (crypt > 0 && !match(parv[crypt], n->crypt)) || (crypt < 0 && match(parv[-crypt], n->crypt)))
#endif
				&& (!join || (join > 0 && getjoininfo(n, parv[join])) || (join < 0 && !getjoininfo(n, parv[-join])))
				&& (++count <= MAXMATCHES || nolimit))
					csntc(nick, "Matching Nick: [%d] \2%s%s\2 -> %s@%s", count, GetPrefix(n), n->nick, n->ident, n->host);

		gettimeofday(&tv, NULL);
		if(count > MAXMATCHES && !nolimit)
			csntc(nick, "[%d] Trop de nicks trouvs, restreignez votre recherche ou forcez avec '-all' en DERNIER argument.", MAXMATCHES);

		csntc(nick, "Fin de la Recherche: %d matche%s trouve%s. (Effectue en %6.3f ms)",
			count, PLUR(count), PLUR(count), (float) (tv.tv_usec - ticks)/1000);
	}

	else if(!strcasecmp(arg, "suspend"))
	{
		if(!(u = ParseNickOrUser(nick, user))) return 0;

		if(nick->user == u) return csntc(nick, "Vous ne pouvez pas vous suspendre.");

		if(IsAnAdmin(u) && u->level >= nick->user->level)
			return csntc(nick, GetReply(nick, L_GREATERLEVEL), u->nick);

		if(!IsSuspend(u) && ((parc == 3 && *parv[3] == '%') || parc < 3))
			return csntc(nick, "Veuillez prciser une raison pour suspendre cet UserName.");

		if(!IsSuspend(u))
		{
			time_t timeout = 0;

			if(*parv[3] == '%' && (timeout = convert_duration(++parv[3])) <= 0)
				return csntc(nick, "Dure incorrecte, le format est %%<XjXhXm> X tant le nombre respectif d'units.");

			do_user_suspend(u, parv2msg(parc, parv, timeout ? 4 : 3, 250), nick->user->nick, timeout, time(NULL));
			show_ususpend(nick, u);

			if((n = u->n))
			{
				csntc(n, "Votre Username vient d'tre suspendu par l'Admin %s.", nick->nick);
				cs_account(n, NULL);
			}
		}
		else
		{
			if(getoption("-nolog", parv, parc, 2, -1))
			{
				free(u->suspend);
				u->suspend = NULL;
			}
			else u->suspend->expire = -1;

			csntc(nick, "L'Username \2%s\2 n'est plus suspendu.", u->nick);
		}
		write_anuser(u);
	}

	else if(!strcasecmp(arg, "cantregchan"))
	{
		if(!strcasecmp(nick->user->nick, user))
			return csntc(nick, "Vous ne voulez plus pouvoir enregistrer de chan ?");

		if(!(u = ParseNickOrUser(nick, user))) return 0;

		if(IsAnAdmin(u))
			return csntc(nick, "Vous ne pouvez pas empecher un Admin d'enregistrer des salons.");

		if(!CantRegChan(u))
		{
			time_t timeout = 0;

			if(parc >= 3 && *parv[3] == '%' && (timeout = convert_duration(++parv[3])) <= 0)
				return csntc(nick, "Dure incorrecte, le format est %%<XjXhXm> X tant le nombre respectif d'units.");

			u->cantregchan = timeout ? (time(NULL) + timeout) : 0;
			csntc(nick, "%s ne peut pas enregistrer de salon%s%s.", u->nick,
				(u->cantregchan > 0) ? " jusqu'au " : "", (u->cantregchan > 0) ? get_time(u->cantregchan) : "");
			if((n = u->n))
			{
				if(timeout)
					csntc(n, "Vous ne pouvez plus enregistrer de salon jusqu'au %s.", get_time(u->cantregchan));
				else
					csntc(n, "Vous ne pouvez plus enregistrer de salon jusqu' ce qu'un admin vous le permette.");
			}
		}
		else
		{
			u->cantregchan = -1;
			csntc(nick, "L'Username \2%s\2 peut maintenant enregistrer des salons.", u->nick);
		}
		write_anuser(u);
	}

	else if(!strcasecmp(arg, "mail"))
	{
		if(parc < 3 || !IsValidMail(parv[3]))
			return csntc(nick, GetReply(nick, L_MAIL_INVALID));

		if(!(u = ParseNickOrUser(nick, user))) return 0;

		Strncpy(u->mail, parv[3], MAILLEN);
		csntc(nick, GetReply(nick, L_OKCHANGED));
		write_anuser(u);
	}

	else if(!strcasecmp(arg, "newpass"))
	{
		char p[15];
		if(parc < 3 || strlen(parv[3]) < 6) return csntc(nick, GetReply(nick, L_PASS_LIMIT));

		if(!(u = ParseNickOrUser(nick, user))) return 0;

		if(IsAnAdmin(u) && u->level >= nick->user->level && u != nick->user)
			return csntc(nick, GetReply(nick, L_GREATERLEVEL), u->nick);

		Strncpy(p, parv[3], sizeof p - 1);
		strncpy(u->passwd, mkpass(p), 20);
		csntc(nick, GetReply(nick, L_OKCHANGED));
		write_anuser(u);
	}

	else if(!strcasecmp(arg, "newnick"))
	{
		char *newusr;
		unsigned int hash;

		if(parc < 3 ) return csntc(nick, "Syntaxe: %s NEWNICK <username> <new>", RealCmd("user"));

		if(strlen(parv[3]) > NICKLEN || !IsValidNick(parv[3]))
			return csntc(nick, GetReply(nick, L_USER_INVALID), parv[3]);

		if(getuserinfo((newusr = parv[3]))) return csntc(nick, GetReply(nick, L_ALREADYREG), newusr);
		if(!(u = getuserinfo(user))) return csntc(nick, GetReply(nick, L_NOSUCHUSER), user);

		if(IsAnAdmin(u) && u->level >= nick->user->level && u != nick->user)
			return csntc(nick, "Vous ne pouvez pas changer l'username d'un Admin de niveau suprieur ou gal.");

		File_UserDel(u->nick);
		hash_deluser(u);
		Strncpy(u->nick, newusr, NICKLEN);
		u->next = user_tab[(hash = do_hashu(newusr))];
		user_tab[hash] = u;
		if(u->n) cs_account(u->n, newusr);
		csntc(nick, GetReply(nick, L_OKCHANGED));
		write_anuser(u);
		write_userlist();
	}

	else if(!strcasecmp(arg, "del"))
	{
#ifdef HAVE_TRACK
		struct track *track;
#endif
		if(!strcasecmp(nick->user->nick, user))
			return csntc(nick, "Vous ne devriez pas vous deluser.. (Utilisez la commande \2%s\2)", RealCmd("drop"));

		if(!(u = ParseNickOrUser(nick, user))) return 0;

		if(IsAnAdmin(u) && u->level >= nick->user->level)
			return csntc(nick, GetReply(nick, L_GREATERLEVEL), u->level);

#ifdef HAVE_TRACK
		if((track = istrack(u)))
		{
			csntc(track->tracker, "[\2Track\2] L'UserName %s que vous trackez vient d'tre supprim par %s.",
				track->tracked->nick, nick->user->nick);
			del_track(u);
		}
#endif
		if((n = u->n))
		{
			char raison[200] = {0};
			if(parc > 2) parv2msgn(parc, parv, 3, raison, sizeof raison);

			cs_account(n, NULL);
			csntc(n, "Votre Username vient d'tre supprim par l'Administrateur %s %s", nick->nick, raison);
		}

		csntc(nick, "L'username %s a bien t supprim.", u->nick);

		for(a = u->accesshead;a;a = a2)
		{
			a2 = a->next;
			if(a->level == OWNERLEVEL)
			{
				char tmp[400];
				snprintf(tmp, sizeof tmp, "Salon %s desenregistr par %s@%s (\2Suppression de l'username de l'owner\2)",
					a->c->chan, nick->nick, nick->user->nick);
				del_chan(a->c, tmp);
			}
		}
		del_regnick(u);
		write_userlist();
	}
#ifdef SQLLOG
	else if(!strcasecmp(arg, "historique"))
	{
		time_t date = 0;
		MYSQL_RES *result;
		MYSQL_ROW row;
		int cmd, chan;
		char tmp[150] = "SELECT * FROM usercmdslog WHERE user='%s' AND TS>=%T";

		if(!(u = ParseNickOrUser(nick, user))) return 0;
		if(parc < 2) return csntc(nick, "Syntaxe: %s HISTORIQUE <username> [%%XjXh] [-cmd <cmd> -chan]", RealCmd("user"));
		if(parc >= 3 && *parv[3] == '%' && (date = convert_duration(++parv[3])) <= 0)
			return csntc(nick, "Dure incorrecte, le format est %%<XjXhXm> X tant le nombre respectif d'units.");

		if((chan = getoption("-chan", parv, parc, 2, -1))) strcpy(tmp, "SELECT * FROM chancmdslog WHERE user='%s' AND TS>=%T");
		if((cmd = getoption("-cmd", parv, parc, 2, 0))) strcat(tmp, " AND cmd='%s'");

		if(sql_query(tmp, u->nick,	date ? time(NULL) - date : 0, cmd ? parv[cmd] : "") < 0)
			return csntc(nick, "La requte SQL a chou.");

		if(!(result = mysql_store_result(bot.sql_id)) || !(row = mysql_fetch_row(result)))
			return csntc(nick, "Aucune entre trouve.");

		if(date) csntc(nick, "Commandes effectues par %s depuis %s.", u->nick, duration(date));
		else csntc(nick, "Commandes effectues par %s.", u->nick);
		for(;row;row = mysql_fetch_row(result))
		{
			if(chan) csntc(nick, "%s: \2%s\2 [%s] sur \2%s", get_time(strtoul(row[2], NULL, 10)), row[3], NONE(row[4]), row[1]);
			else csntc(nick, "%s: commande \2%s\2 [%s]", get_time(strtoul(row[1], NULL, 10)), row[2], NONE(row[3]));
		}
		mysql_free_result(result);
	}
#endif
	else if(!strcasecmp(arg, "stats"))
	{
		int memn = getoption("-mem", parv, parc, 2, -1);
		int cmds = getoption("-cmds", parv, parc, 2, 0);
		int traffic = getoption("-traffic", parv, parc, 2, -1);
		int all = getoption("-all", parv, parc, 2, -1);

		if(memn || all)
		{ /*memm: mmoire totale, usr: nb d'usernames, chs: nb de chans reg, usra: nb de nicks logus*/
			int memm = 0, usr = 0, chs = 0, usra = 0, ni = 0, maxbans, maxaccess, ma, mb, i = 0;
			int maxb = 0, tmpb = 0, useb = 0, bb = 0;/* Hash stats */
			aBan *ban;
			aJoin *j;
			struct badnicks *bn = badnhead;
			struct badchans *bc = badchead;
			struct ignore *ig = ignorehead;
			aLink *lp;

			if(!(maxbans = getoption("-maxbans", parv, parc, 2, 1))) maxbans = WARNACCESS;
			if(!(maxaccess = getoption("-maxaccess", parv, parc, 2, 1))) maxaccess = WARNACCESS;

			uptime(nick, chaninfo, parc, parv);

			for(bb = 0, maxb = 0, useb = 0;i < NICKHASHSIZE;i++)
			{
				for(tmpb = 0, n = nick_tab[i];n;n = n->next, ni++)
				{
					tmpb++;/* item dans la petite linkedlist*/
					for(ma = 0, j = n->joinhead;j;j = j->next, ma++);
					memm += ma * sizeof(aJoin);
				}
				if(tmpb > maxb) maxb = tmpb;/* linkedlist locale la + longue?*/
				useb += tmpb;
				if(tmpb) bb++;/* une linked liste utilise en +*/
			}
			csntc(nick, "Hash Nick: Offsets (Utiliss / Dispos) %d / %d (Max. %d Av. %6.3f)", bb, NICKHASHSIZE, maxb, (float) useb / bb);

			for(i = 0, bb = 0, maxb = 0, useb = 0;i < USERHASHSIZE;i++)
			{
				for(tmpb = 0, u = user_tab[i];u;u = u->next, usr++, tmpb++)
				{
					if(u->suspend) memm += sizeof(struct suspendinfo);
					if(u->lastlogin) memm += strlen(u->lastlogin);
					if(u->n) usra++;
				}
				if(tmpb > maxb) maxb = tmpb;
				useb += tmpb;
				if(tmpb) bb++;
			}
			csntc(nick, "Hash User: Offsets (Utiliss / Dispos) %d / %d (Max. %d Av. %6.3f)", bb, USERHASHSIZE, maxb, (float) useb / bb);

			for(i = 0, bb = 0, maxb = 0, useb = 0;i< CHANHASHSIZE;i++)
			{
				for(tmpb = 0, chaninfo = chan_tab[i];chaninfo;chaninfo = chaninfo->next, chs++, tmpb++)
				{
					if(chaninfo->suspend) memm += sizeof(struct suspendinfo);
					if(chaninfo->motd) memm += strlen(chaninfo->motd);

					for(mb = 0, ban = chaninfo->banhead;ban;ban = ban->next, mb++) if(ban->raison) memm += strlen(ban->raison);
					if(mb >= maxbans) csntc(nick, "Warning Bans Limite: \2\00304%d\2\3 Bans sur %s", mb, chaninfo->chan);

					for(ma = 0, lp = chaninfo->access;lp;lp = lp->next, ma++) if(lp->value.a->info) memm += strlen(lp->value.a->info);
					if(ma >= maxaccess) csntc(nick, "Warning Acces Limite: \2\00304%d\2\3 Accs sur %s", ma, chaninfo->chan);
					memm += ma * (sizeof(anAccess) + sizeof(aLink)) + mb * sizeof(aBan) + chaninfo->users * sizeof(aLink);
				}
				if(tmpb > maxb) maxb = tmpb;
				useb += tmpb;
				if(tmpb) bb++;
			}
			csntc(nick, "Hash Chan: Offsets (Utiliss / Dispos) %d / %d (Max. %d Av. %6.3f)", bb, CHANHASHSIZE, maxb, (float) useb / bb);

			for(ma = 0;bn;bn = bn->next, ma++); memm += ma * sizeof(struct badnicks);
			for(ma = 0;bc;bc = bc->next, ma++); memm += ma * sizeof(struct badchans);
			for(ma = 0;ig;ig = ig->next, ma++); memm += ma * sizeof(struct ignore);
			memm += ni * sizeof(aNick) + usr * sizeof(anUser) + chs * sizeof(aChan) +
				(USERHASHSIZE + CHANHASHSIZE + NICKHASHSIZE + MAXNUM) * sizeof(void *);

			csntc(nick, "Memoire Utilise pour les Users/Chans: %10.3f KB", memm / 1024.0);
			csntc(nick, "Il y a %d Usernames et %d Chans enregistrs.", usr, chs);
			csntc(nick, "%d Chatteurs connects, dont %4.1f %% logus.", ni, (usra / (float) ni) * 100);
		}
		if(cmds || all)
		{
			int x = 0, adm = 0, usr = 0;

			for(;x < cs_cmds_size;x++)
			{
				if(AdmCmd(x)) adm += cs_cmds[x].used;
				else usr += cs_cmds[x].used;
				if(cmds && !match(parv[cmds],cs_cmds[x].cmd) && cs_cmds[x].cmd[0] != '\1')
					csntc(nick, "- %s (\002%d\2)", cs_cmds[x].cmd, cs_cmds[x].used);
			}
			csntc(nick, "Un total de %d commandes utilises, dont %d Admins (Ratio: %5.2f%)",
				(adm + usr), adm, (adm / (float) (usr + adm)) * 100);
		}
		if(traffic || all)
		{
			float up, down;
			int mbu = 0, mbd = 0;

			up = bot.dataS/1024.0;
			down = bot.dataQ/1024.0;
			if(up >= 1024)
			{
				up /= 1024.0;
				mbu = 1;
			}
			if(down >= 1024)
			{
				down /= 1024.0;
				mbd = 1;
			}
			csntc(nick, "Traffic: Up: \2%10.3f %s\2 Down: \2%10.3f %s\2", up, mbu ? "MB":"KB", down, mbd ? "MB":"KB");
			csntc(nick, "Traffic Actuel: %10.3f Ko/s", (float) (bot.dataQ - bot.lastbytes) / ((time(NULL) - bot.lasttime) * 1024.0));
		}
		if(!all && !memn && !traffic && !cmds)
			csntc(nick, GetReply(nick, L_UNKNOWNOPTION), parc < 2 ? "<NULL>" : parv[2]);
	}
	else if(!strcasecmp(arg, "nopurge"))
	{
		if(!(u = ParseNickOrUser(nick, user))) return 0;

		switch_option(nick, parc < 3 ? NULL : parv[3], arg, u->nick, &u->flag, U_NOPURGE);
		write_anuser(u);
	}
	else if(!strcasecmp(arg, "checkpass"))
	{
		if(parc < 3)
			return csntc(nick, "Syntaxe: %s CHECKPASS <username|%%nick> <pass>", RealCmd("user"));

		if(!(u = ParseNickOrUser(nick, user))) return 0;

		if(!checkpass(parv[3], u)) csntc(nick, "Le mot de passe n'est \2PAS\2 correct !");
		else csntc(nick, "Le mot de passe \2EST\2 correct.");
	}
	else return csntc(nick, GetReply(nick, L_UNKNOWNOPTION), arg);
	return 1;
}

void do_user_suspend(anUser *user, const char *raison, const char *nick, time_t expire, time_t debut)
{
	if(!user->suspend) user->suspend = calloc(1, sizeof(struct suspendinfo));
	Strncpy(user->suspend->raison, raison, 250);
	Strncpy(user->suspend->from, nick, NICKLEN);
	user->suspend->debut = debut;
	user->suspend->expire = expire;
	return;
}

void show_ususpend(aNick *nick, anUser *user)
{
	if(!user->suspend) return; /* on aurait jamais du l'envoyer l*/
	if(IsSuspend(user))
	{
		char buf[200] = {0};
		Strncpy(buf, get_time(user->suspend->debut), sizeof buf - 1);
		csntc(nick, "L'UserName %s \2EST\2 suspendu par %s (le %s). Expire %s", user->nick, user->suspend->from, buf,
			user->suspend->expire ? get_time((user->suspend->debut + user->suspend->expire)) : "Jamais");
	}
	else csntc(nick, "L'UserName %s \2ETAIT\2 suspendu par %s", user->nick, user->suspend->from);
	csntc(nick, "Raison: %s", user->suspend->raison);
	return;
}

int show_userinfo(aNick *nick, anUser *user, int flag)
{
	anAccess *a;
	char chans[451] = {0}, chansw[451] = {0}, b[REGCHANLEN + 20];
	int size = 0, firstt = 0, sizew = 0, firstw = 0;

	csntc(nick, GetReply(nick, L_INFO_ABOUT), user->nick);
	csntc(nick, GetReply(nick, L_MAILIS), user->mail);

	if(UFirst(user)) csntc(nick, " En cours de validation, enregistrement il y a %s", duration(time(NULL) - user->lastseend));
	else csntc(nick, GetReply(nick, L_LASTLOGINTIME), duration(time(NULL) - user->lastseend));
	if(IsAnAdmin(user)) csntc(nick, GetReply(nick, L_USERISADMIN), user->level);

	for(a = user->accesshead;a;a = a->next)
	{
		int p = sprintf(b, "%s (%d) ", a->c->chan, a->level);
		if(AWait(a))
		{
			int tmps = sizew;
			sizew += p;
			if(sizew >= sizeof chansw -1) /* le dernier fait depasser le buffer */
			{
				csntc(nick, !firstw ? GetReply(nick, L_PROPACCESS) : "%s ...", chansw);
				strcpy(chansw, b); /* ajoute le suivant pour la prochaine*/
				firstw = 1;
				sizew = p;
			}
			else strcpy(&chansw[tmps], b);
		}
		else
		{
			int tmps = size;
			size += p;
			if(size >= sizeof chans -1) /* le dernier fait depasser le buffer */
			{
				csntc(nick, !firstt ? GetReply(nick, L_ACCESSON) : "%s ...", chans);
				strcpy(chans, b); /* ajoute le suivant pour la prochaine*/
				firstt = 1;
				size = p;
			}
			else strcpy(&chans[tmps], b);
		}
	}

	if(*chans) csntc(nick, !firstt ? GetReply(nick, L_ACCESSON) : "%s", chans);
	else if(!firstt) csntc(nick, GetReply(nick, L_NOACCESS));
	if(*chansw) csntc(nick, !firstw ? GetReply(nick, L_PROPACCESS) : "%s", chansw);
	else if(!firstw) csntc(nick, GetReply(nick, L_NOPROPACCESS));

	if(flag && user->n) csntc(nick, " %s%s est actuellement logu sous cet Username (%s@%s)",
		GetPrefix(user->n), user->n->nick, user->n->ident, user->n->host);
	else if(flag) csntc(nick, " Personne n'est actuellement logu sous cet username");

	if(user->suspend) show_ususpend(nick, user);
	csntc(nick, GetReply(nick, L_OPTIONS), GetUserOptions(user));
	csntc(nick, " Last Login depuis %s", user->lastlogin ? user->lastlogin : "<Non Disponible>");

#ifdef HAVE_VOTE
	if(!flag && vote_actif)
	{
		if(nick->user->vote) csntc(nick, " Vous avez vot pour: \2%s", vote[user->vote -1].prop);
		else csntc(nick, " Vous n'avez pas encore vot");
	}
#endif
	return 0;
}
