/* src/cs_register.c - Enregistrement de salon
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: cs_register.c,v 1.40 2003/12/27 16:19:25 romexzf Exp $
 */

#include "main.h"
#include "hash.h"
#include "add_info.h"
#include "fichiers.h"
#include "outils.h"
#include "cs_cmds.h"

/*
 * register_chan <salon> <description>
 */
int register_chan(aNick *nick, aChan *c, int parc, char **parv)
{
	char *salon = parv[1], description[200] = {0}, *ptr;
	aJoin *chan;

	if(!chan_check(salon, nick)) return 0;

	if(CantRegChan(nick->user))
		return csntc(nick, "Vous ne pouvez pas enregistrer de salon%s%s.",
				(nick->user->cantregchan > 0) ? " jusqu'au " : "", (nick->user->cantregchan > 0) ? get_time(nick->user->cantregchan) : "");

	parv2msgn(parc, parv, 2, description, 200);

	if(strlen(description) < 12)
		return csntc(nick, GetReply(nick, L_MINDESCRIPTIONLEN));

	if((ptr = IsAnOwner(nick->user)))
		return csntc(nick, GetReply(nick, L_ALREADYOWNER), ptr);

	if(getchaninfo(salon)) return csntc(nick, GetReply(nick, L_ALREADYREG), salon);

	if(!(chan = getjoininfo(nick, salon)) || !IsOp(chan))
		return csntc(nick, GetReply(nick, L_NEEDTOBEOP), salon);

	c = add_chan(salon, description);
	add_access(nick->user, salon, OWNERLEVEL, A_OP | A_PROTECT, time(NULL));

	csjoin(c);
	putlog(LOG_CHANS, "REGCHAN %s par %s@%s (%s)", salon, nick->nick, nick->user->nick, description);
	csntc(nick, GetReply(nick, L_CHANNOWREG), salon);

	append_file(CHANS_FILE,c->chan);
	write_achan(c);
	write_anuser(nick->user);
	return 1;
}

/*
 * ren_chan <chan> <nouveau chan>
 */
int ren_chan(aNick *nick, aChan *c, int parc, char **parv)
{
	aChan *c2;
	aLink *acces;
	aJoin *j;
	char *newchan = parv[2], path1[250] = DBDIR "/chans/", path2[250] = DBDIR "/chans/";
	unsigned int hash = do_hashc(newchan), len = strlen(path1);

	if(c->flag & C_ALREADYRENAME && !IsAnAdmin(nick->user))
		return csntc(nick, GetReply(nick, L_CMDALREADYUSED), parv[0]);

	if(!chan_check(newchan, nick)) return 0;

	if((c2 = getchaninfo(newchan)))
		return csntc(nick, GetReply(nick, L_ALREADYREG), c2->chan);

	if(!IsAnAdmin(nick->user) && (!(j = getjoininfo(nick, newchan)) || !IsOp(j)))
		return csntc(nick, GetReply(nick, L_NEEDTOBEOP), newchan);

	cspart(c, newchan);

	putlog(LOG_CHANS, "RENCHAN %s %s par %s@%s", c->chan, newchan, nick->nick, nick->user->nick);
	hash_delchan(c);
	Strncpy(c->chan, newchan, REGCHANLEN);
	c->next = chan_tab[hash];/* swap le vieux*/
	chan_tab[hash] = c;		/* et le nouveau (mis en tte)*/
	csjoin(c);
	write_chanlist();

	for(acces = c->access;acces;acces = acces->next)
		write_anuser(acces->value.a->user);

	c->flag |= C_ALREADYRENAME;
	conv_char(parv[1], path1 + len);
	conv_char(newchan, path2 + len);
	rename(path1, path2);

	csntc(nick, GetReply(nick, L_OKCHANGED));
	return 1;
}

/*
 * unreg_chan salon raison
 */
int unreg_chan(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *salon = parv[1], tmp[400];

	if(!nick->user || ChanLevelbyUserI(nick->user, chaninfo) != OWNERLEVEL)
		return csntc(nick, GetReply(nick, L_NEEDTOBEOWNER));

	if(parc < 2 || strcasecmp(parv[parc], "CONFIRME"))
		return csntc(nick, GetReply(nick, L_NEEDCONFIRM));

	csntc(nick, GetReply(nick, L_CHANUNREG), salon);
	snprintf(tmp, sizeof tmp, "Salon %s desenregistr par %s@%s (%s)", salon, nick->nick, nick->user->nick,
		parc > 2 ? parv2msg(parc - 1, parv, 3, 150) : "Aucune raison");

	if(CANTREGONUNREG) nick->user->cantregchan = time(NULL) + CANTREGONUNREG;
	del_chan(chaninfo, tmp);
	return 1;
}
