/* src/hash_user.c - gestion des hash user
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: hash_user.c,v 1.8 2003/12/27 17:54:20 romexzf Exp $
 */

#include "main.h"
#include "hash.h"
#include "debug.h"
#include "outils.h"
#include "fichiers.h"
#include "del_info.h"

int hash_deluser(anUser *user)
{
	unsigned int hash = do_hashu(user->nick);
	anUser *tmp = user_tab[hash];

	if(tmp == user) user_tab[hash] = user->next;
	else
	{
		for(;tmp && tmp->next != user;tmp = tmp->next);
		if(tmp) tmp->next = user->next;
		else Debug(W_WARN|W_MAX, "H_del_user %s non trouv  l'offset %u ?!", user->nick, hash);
	}
	return 0;
}

int CountUsers(const char *mask)
{
	int i = 0, res = 0;
	anUser *u;

	for(;i < USERHASHSIZE;i++) for(u = user_tab[i];u;u = u->next) if(!mask || !match(mask, u->nick)) res++;
	return res;
}

anUser *getuserinfo(const char *nick)
{
	unsigned int hash = do_hashu(nick);
	register anUser *tmp = user_tab[hash];
	for(;tmp && strcasecmp(nick, tmp->nick);tmp=tmp->next);
	return tmp;
}

anUser *add_regnick(const char *user, const char *pass, time_t lastseend, int level, int flag, const char *mail, int vote)
{
	anUser *new;
	unsigned int hash = do_hashu(user);

	if(!(new = calloc(1, sizeof(anUser))))
	{
		Debug(W_MAX|W_WARN, "add_regnick, malloc a chou pour anUser %s", user);
		return NULL;
	}
	if(!(user && pass && mail && level))
	{
		Debug(W_MAX, "add_regnick appel avec trop peu d'arguments user=%s pass=%s mail=%s level=%d",
					NONE(user), NONE(pass), NONE(mail), level);
		free(new);
		return NULL;
	}
	Strncpy(new->nick, user, NICKLEN);
	strcpy(new->passwd, pass);
	new->lastseend = lastseend;
	new->level = level;
	Strncpy(new->mail, mail, MAILLEN);
#ifdef HAVE_VOTE
	new->vote = vote;
#endif
	new->flag = flag;
	new->suspend = NULL;
	new->lastlogin = NULL;
	new->cantregchan = -1;

	new->next = user_tab[hash];
	user_tab[hash] = new;
	return new;
}

void del_regnick(anUser *user)
{
	anAccess *chan, *ct;
	int writechan = 0;
#ifdef USE_MEMOSERV
	aMemo *memo = user->memohead, *mem;
#endif

	hash_deluser(user);
	File_UserDel(user->nick);

	for(chan = user->accesshead;chan;chan = ct)
	{
		ct = chan->next;
		if(chan->level == OWNERLEVEL)
		{
			del_chan(chan->c, "delregnick de l'owner");
			writechan = 1;
		}
		else del_access(user, chan->c);
	}
#ifdef USE_MEMOSERV
	for(;memo;memo = mem)
	{
		mem = memo->next;
		free(memo);
	}
#endif
	if(user->suspend) free(user->suspend);
	free(user);
	write_userlist();
	if(writechan) write_chanlist();
}
