/* src/serveur.c - Traitement des messages IRC
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: serveur.c,v 1.170 2003/12/29 19:36:30 romexzf Exp $
 */

#include "main.h"
#include <unistd.h>
#include "outils.h"
#include "serveur.h"
#include "add_info.h"
#include "hash.h"
#include "flood.h"
#include "cs_cmds.h"
#include "config.h"
#include "del_info.h"
#include "debug.h"
#include "aide.h"
#include <sys/time.h>
#ifdef USE_MEMOSERV
#include "memoserv.h"
#endif
#ifdef USE_WELCOMESERV
#include "welcome.h"
#endif
#ifdef HAVE_TRACK
#include "track.h"
#endif
#ifdef SQLLOG
#include "sql_log.h"
#endif

aServer *mainhub = NULL;
static int burst = 0;

#ifndef N2K
/*
 * m_account SS|SSCCC AC SSCCC account|""
 */
int m_account(int parc, char **parv)
{
	aNick *nick = NULL;
	anUser *user;

	if(parc < 3 || !(nick = num2nickinfo(parv[1])))
		return Debug(W_DESYNCH|W_PROTO|W_WARN, "Erreur dans un TOKEN AC de %s: arg=%d nick=%s", parv[0], parc, nick ? nick->nick : "NULL");

	if(!(user = getuserinfo(parv[2])) || user->n)
	{
		aServer *links = num2servinfo(parv[0]);
		cswallops("ACCOUNT %s (%s) pour %s depuis %s", user ? "Desynch" : "inconnu", parv[2], nick->nick, links->serv);
		cs_account(nick, NULL); /* desacounter :)*/
		return 0;
	}

	nick->user = user;
	nick->user->n = nick;
	nick->user->lastseend = time(NULL);
	csntc(nick, "Vous avez t \2ridentifi\2");
#ifdef USE_NICKSERV
	if(IsProtected(user)) del_killinfo(nick, user->nick); /* arret du kill */
#endif
	return 0;
}

/*
 * m_setident SCC SETIDENT ident
 */
int m_setident(int parc, char **parv)
{
	aNick *who = num2nickinfo(parv[0]);

	if(parc > 1 && who) Strncpy(who->ident, parv[1], USERLEN);
	return 0;
}

/*
 * m_sethost SCC SETHOST host.com
 */
int m_sethost(int parc, char **parv)
{
	aNick *who = num2nickinfo(parv[0]);

	if(parc < 2 || !who || !*parv[1] || strchr(parv[1], ':') || !strchr(parv[1], '.')) return 0;
#ifdef HAVE_CRYPTHOST
	Strncpy(who->crypt, parv[1], HOSTLEN);
#else
	Strncpy(who->host, parv[1], HOSTLEN);
#endif
	return 0;
}

#else/* N2K*/

/*
 * m_clearmode SSCCC CM # mode_cleared
 */
int m_clearmode(int parc, char **parv)
{
	aChan *chan = getchaninfo(parv[1]);
	int clear = 0;

	if(strchr(parv[2], 'o')) clear |= J_OP;
	if(strchr(parv[2], 'v')) clear |= J_VOICE;

	if(chan && !IsSuspend(chan))
	{
		aLink *j;
		char mode[30] = "-";

		Strncpy(mode + 1, parv[2], sizeof mode -2);
		string2scmode(&chan->mode, mode, NULL, NULL);

		if(!clear) return 0; /* aucun changement de +o/+v*/
		for(j = chan->members;j;j = j->next)
			 if(!(j->value.j->nick->flag & (N_SERVICE | N_GOD))) j->value.j->status &= ~clear;
	}
	else
	{
		register aNick *nick;
		aJoin *join;
		int i = 0;

		if(!clear) return 0; /* aucun changement de +o/+v*/
		for(;i < NICKHASHSIZE;i++)
			for(nick = nick_tab[i];nick;nick = nick->next)
				if(!(nick->flag & (N_SERVICE|N_GOD)) && (join = getjoininfo(nick, parv[1])))
					join->status &= ~clear;
	}
	return 0;
}
#endif

/*
 * m_mode SSCCC M #|SSCCC +|-m
 */
int m_mode(int parc, char **parv)
{
	aNick *nick, *nicktemp = NULL;

	if(parc < 3) return Debug(W_PROTO|W_WARN, "#arg invalide: MODE de %s: #arg=%d", parv[0], parc);
	if(*parv[1] == '#')
	{
		aChan *ch = getchaninfo(parv[1]);
		char *modes = parv[2], *param = NULL, *key = NULL, *limit = NULL;
		anAccess *a = NULL, *a2 = NULL;
		aJoin *join = NULL;
		aLink *j = NULL;
		int what = 1, i = 2, l = 0;

		if(ch && IsSuspend(ch)) ch = NULL;
		while (*modes)
		{
			switch (*modes)
			{
				case '+':
					what = 1;
					break;
				case '-':
					what = 0;
					break;
				case 'o':
					param = parv[++i];
					nicktemp = NULL;

					if(ch) /* salon reg */
					{
						if(!what) /* -o */
						{
							if((nicktemp = num2nickinfo(param)) && nicktemp->user &&
							(nick = GetInfobyPrefix(parv[0])) /* la victime a un accs ou est protge*/
							&& nick != nicktemp && (IsAnAdmin(nicktemp->user) || (a = GetAccessIbyUserI(nicktemp->user, ch))))
							{
								if(!nick->user || (nicktemp->user->level > nick->user->level ||/* l'OP n'est pas auth ou lvl infrieur*/
								(a && AProtect(a) && !ASuspend(a) && (a2 = GetAccessIbyUserI(nick->user, ch))
								&& !ASuspend(a2) && a2->level <= a->level)))/* ou accs infrieur/suspend*/
								{
									csmode(ch, MODE_OBV, "+o %s", nicktemp->numeric);
									break;
								}
							}
							else if(!strcmp(cs.num, param)) /* deop du CServ */
							{
								csmode(ch, MODE_OBV, "+o %s", cs.num);
								break;
							}
						}
						else /* +o */
						{ /* noops ou strictop */
							if((CNoOps(ch) && (nicktemp = num2nickinfo(param)) && !IsOper(nicktemp))
								|| (CStrictOp(ch) && nicktemp && (!nicktemp->user || !(a = GetAccessIbyUserI(nicktemp->user, ch)) || ASuspend(a))))
							{
								csmode(ch, MODE_OBV, "-o %s", param);
								break;
							}
						}

					} /* mise  jour du status */

					if((nicktemp || (nicktemp = num2nickinfo(param))) /*"cache"*/
							&& (join = getjoininfo(nicktemp, parv[1])))
					{
						if(what) DoOp(join);
						else DeOp(join);
					}

					break;

				case 'v':
					param = parv[++i];  /* mise  jours des status*/

					if((nicktemp = num2nickinfo(param)) && (join = getjoininfo(nicktemp, parv[1])))
					{
						if(what) DoVoice(join);
						else DeVoice(join);
					}
					break;

				case 'b':
					param = parv[++i]; /*l on modifie 'i' pour qu'il passe au paramtre de ce mode*/

					if(!what || !ch) break;/* salon non reg, ou -b, on ignore, si ban perm, il sera enforc*/

					if(!CNoBans(ch))
					{
						if(!(nicktemp = GetInfobyPrefix(parv[0]))) break;
						l = nicktemp->user ? ChanLevelbyUserI(nicktemp->user, ch) : 0;
						if(!ch->banlevel || l >= ch->banlevel)/* peut  priori bannir*/
						{
							for(j = ch->members;j;j = j->next)
							{/* recherche de personnes protges couvertes*/
								nick = j->value.j->nick;
								if(!nick->user || match(param, GetNUHbyNick(nick, 0))) continue;
								if((a = GetAccessIbyUserI(nick->user, ch)) && AProtect(a) && !ASuspend(a)) break;
								else a = NULL;
							}
						}
					}

					if(CNoBans(ch) || ((ch->banlevel > l || a) && (!nicktemp->user || !IsAnAdmin(nicktemp->user))))
						csmode(ch, MODE_OBV, "-b %s", param);

					break;

				case 'l':
					if(what) limit = parv[++i];
					break;
				case 'k':
					key = parv[++i];
					break;
			}
			modes++;
		}
		if(ch)
		{ /*test du ChangeModesLevel (et action..), un peu lourd mais efficace*/
			if((nicktemp = GetInfobyPrefix(parv[0])) &&
				ch->cml && nicktemp->user && !IsAnAdmin(nicktemp->user) &&
				(!(a = GetAccessIbyUserI(nicktemp->user, ch)) || ASuspend(a) || a->level < ch->cml))
			{
				char new[30] = {0}, tmp[20];
				int f = 0, nk = 0;
				modes = parv[2];
				while (*modes)
				{
					if(*modes != 'b' && *modes != 'v' && *modes != 'o')
					{
						if(*modes == '+') new[f] = '-';
						else if(*modes == '-') new[f] = '+';
						else if(*modes == 'k') nk = 1;
						else new[f] = *modes;
						f++;
					}
					modes++;
				}
				snprintf(tmp, 20, "%d", ch->mode.limit);
				if(new[1] != '\0') putserv("%s " TOKEN_MODE " %s %s %s", cs.num, ch->chan, new,
					((ch->mode.modes & C_MLIMIT) && (limit || strchr(new, 'l'))) ? tmp : "");
				if(nk) putserv("%s " TOKEN_MODE " %s %ck %s", cs.num, ch->chan, (ch->mode.modes & C_MKEY) ? '+' : '-', (ch->mode.modes & C_MKEY) ? ch->mode.key : key);
			}
			else string2scmode(&ch->mode, parv[2], key, limit);
		}
	}
	else if((nick = getnickbynick(parv[1]))) nick->flag = parse_umode(nick->flag, parv[2]);
	return 0;
}

/*
 * m_part ABAAA L # [:raison]
 */
int m_part(int parc, char **parv)
{
	char *p, *ptr = NULL;
	aNick *nick = NULL;

	if(parc < 2) return Debug(W_PROTO|W_WARN, "#arg invalide: PART de %s: #arg=%d", parv[0], parc);

	if(!(nick = GetInfobyPrefix(parv[0])))
		return Debug(W_DESYNCH|W_WARN, "PART %s de %s: aNick non trouv!", parv[1], parv[0]);

	for(p = Strtok(&ptr, parv[1], ',');p;p = Strtok(&ptr, NULL, ',')) del_join(nick, p);
	return 0;
}

/*
 * m_kick SSCCC K # SSCCC :raison
 */
int m_kick(int parc, char **parv)
{
	aNick *k, *v;
	aChan *ch;
	aJoin *j = NULL;
	anAccess *a1 = NULL, *a2 = NULL;

	if(parc < 3) return Debug(W_PROTO|W_WARN, "#arg invalide: KICK de %s: #arg=%d", parv[0], parc);

	if(!(v = num2nickinfo(parv[2])))
		return Debug(W_DESYNCH|W_WARN, "KICK de %s sur %s par %s: cible introuvable?!", parv[0], parv[2], parv[1]);

	del_join(v, parv[1]);

	if(!(ch = getchaninfo(parv[1])) || IsSuspend(ch)) return 0;

	if(!strcmp(cs.num, parv[2])) /*on sait jamais*/
	{
		if(!strcasecmp(parv[1], bot.pchan))
		{
        	putserv("%s " TOKEN_JOIN " %s", cs.num, parv[2]);
			putserv("%s " TOKEN_MODE " %s +o %s", cs.num, parv[2], cs.num);
		}
		else csjoin(ch);
		return 0;
	}

	if(v->user && (k = GetInfobyPrefix(parv[0])) && k != v /* gnrique*/
		&& ((IsAnAdmin(v->user) && (!k->user || k->user->level < v->user->level))/* v admin >*/
			|| ((a1 = GetAccessIbyUserI(v->user, ch)) && !ASuspend(a1)/* OU v a un accs valide*/
				 && (!k->user  /* et k non logu*/
					 || (!IsAnAdmin(k->user)/* si auth mais PAS admin */
				 		&& (!(a2 = GetAccessIbyUserI(k->user, ch)) || ASuspend(a2)/* OU n'a pas d'accs valide*/
							|| (AProtect(a1) && a2->level <= a1->level))))))) /* OU v protect ET k <= v*/
	{
		csmode(ch, MODE_OBV, "-o %s", k->numeric);/*pas le droit de kick > dop*/
		if((j = getjoininfo(k, ch->chan))) DeOp(j);
	}
	return 0;
}

/*
 * m_join SSCCC J # TS
 */
int m_join(int parc, char **parv)
{
	char *p, *ptr = NULL;
	aNick *nick = NULL;

	if(parc < 2) return Debug(W_PROTO|W_WARN, "#arg invalide: JOIN de %s: arg=%d", parv[0], parc);

	if(!(nick = GetInfobyPrefix(parv[0])))
		return Debug(W_DESYNCH|W_WARN, "JOIN %s de %s: aNick non trouv!", parv[1], parv[0]);

	if(*parv[1] == '0') /* partall !!*/
	{
		aJoin *join = nick->joinhead, *jt;
		for(;join;join = jt)
		{
			jt = join->next;
			del_join(nick, join->channel);
		}
	}
	else for(p = Strtok(&ptr, parv[1], ',');p;p = Strtok(&ptr, NULL, ',')) add_join(nick, p, 0, time(NULL), NULL);
	return 0;
}

/*
 * m_create SSCCC C # TS
 */
int m_create(int parc, char **parv)
{
	char *p, *ptr = NULL;
	aNick *nick;

	if(parc < 3) return Debug(W_PROTO|W_WARN, "#arg invalide: CREATE de %s: arg=%d", parv[0], parc);
	if(!(nick = num2nickinfo(parv[0])))
		return Debug(W_DESYNCH|W_WARN, "CREATE %s de %s: aNick non trouv!", parv[1], parv[0]);

	for(p = Strtok(&ptr, parv[1], ',');p;p = Strtok(&ptr, NULL, ',')) add_join(nick, p, J_OP, atol(parv[2]), NULL);
	return 0;
}

/*
 *m_nick SS N nick (hop) TS ident host.com [+mode [args ...]] IPBASE64 num realname
 */
int m_nick(int parc, char **parv)
{
	char *nick = parv[1], *ac = NULL/*, *spoof = NULL*/;
	aNick *who = NULL;
	anUser *user;
	struct badnicks *badnick;

	if(parc > 7)
	{  /*Nouveau nick*/
						/*  		ident	host	base64			num				ptr to serv						real-name	timestamp		umodes*/
		who = add_nickinfo(nick, parv[4], parv[5], parv[parc-3], parv[parc-2], num2servinfo(parv[0]), parv[parc-1], atol(parv[3]), *parv[6] == '+' ? parv[6] : NULL);
		if(!who) return 0; /* sauvetage temporaire..*/

		if(*parv[6] == '+')
		{
			int i = 7;
			if(who->flag & N_REG) ac = parv[i++];
			/*if(who->flag & N_SPOOF) spoof = parv[i++];*/
		}

		if(ac)
		{
			if(!(user = getuserinfo(ac)) || user->n)
			{
				cswallops("ACCOUNT %s (%s) pour %s depuis %s", ac, user ? "Desynch" : "inconnu", who->nick, who->serveur->serv);
				cs_account(who, NULL);
				return 0;
			}
			who->user = user;
			who->user->n = who;
			who->user->lastseend = time(NULL);
			csntc(who, "Vous avez t \2ridentifi\2");
		}

#ifdef USE_WELCOMESERV /* Envoi du welcome si il est actif et non nul*/
		if(GetConf(CF_WELCOME) && !burst) choose_welcome(who->numeric);
#endif

	}
	else if(parc <= 3)
	{  /* Changement de pseudo */
		unsigned int hash;
		if(!(who = num2nickinfo(parv[0]))) return Debug(W_DESYNCH|W_WARN, "NICK %s > %s: aNick non trouv!", parv[0], nick);
		del_killinfo(who, NULL);
		hash_delnick(who);
		Strncpy(who->nick, nick, NICKLEN);

		hash = do_hashn(nick);
		who->next = nick_tab[hash];
		nick_tab[hash] = who;

	}
	else return Debug(W_PROTO|W_WARN, "#arg invalide: NICK %s %s %s: arg=%d", parv[0], parv[1], parv[2], parc);

	if((badnick = getbadnick(nick)) && !(badnick->flag & CF_NOREG))
	{
		csntc(who, GetReply(who, L_TOOKBADNICK));
		if(!GetConf(CF_CHANGENICK))
			csntc(who, GetReply(who, L_NICKKILLED), kill_interval);
		add_killinfo(who, time(NULL), 1, !GetConf(CF_CHANGENICK));
		return 0;/*on check meme pas le reste..*/
	}
	if(who->user && !strcasecmp(who->user->nick, nick)) return 0;/*le mec est auth sur ce nick > exit*/

#ifdef USE_NICKSERV
	if(!(user = getuserinfo(nick))) return 0;/*le pseudo n'est pas reg, on evite leisregnick > exit*/

	csntc(who, GetReply(who, L_TOOKREGNICK), nick, cs.nick, RealCmd("login"), nick);

	if(IsProtected(user))
	{
		if(UPKill(user)) csntc(who, GetReply(who, L_NICKKILLED), kill_interval);
		else csntc(who, GetReply(who, L_NICKCHANGED), kill_interval);
		add_killinfo(who, time(NULL), 2, !!UPKill(user));
	}
#endif
	return 0;
}

/*
 * m_burst SS B # TS [mode key lim] users %ban
 */
int m_burst(int parc, char **parv)
{
	aNick *nick = NULL;
	aLink *j = NULL;
	aChan *c = NULL;
	char *modes = NULL, *chan = parv[1], *users = NULL;
	char *flags = NULL, *num = NULL, *key = NULL, *limit = NULL;
	int status = 0, i = 4;
	long il = atol(parv[2]);

	if(parc < 4)
		return parc < 3 ? Debug(W_PROTO|W_WARN, "#arg invalide: BURST de %s: arg=%d", parv[0], parc) : 0;
		/* le parc == 3 n'est pas hors protocole (mme si a parait bizzare et que de toute faon on s'en sert pas.
		 * > s'il la liste d'user est vide c'est qu'il y a des zombies */
	if(*parv[3] == '+')
	{  /* channel has mode(s) */
		modes = parv[3];
		if(strchr(modes, 'l')) limit = parv[i++];
		if(strchr(modes, 'k')) key = parv[i++];
		users = parv[i];
	}
	else if(*parv[3] != '%') users = parv[3];
	else return 0; /*bans only*/

	if(!burst)
	{
		if((c = getchaninfo(chan)) && (!c->chanstamp || il < c->chanstamp))
		{ /* les users du chan vont se faire deop et les modes rinitialiss comme le veut le serveur qui envoit le burst*/
			c->mode.modes = 0;
			c->mode.limit = 0;
			*c->mode.key = 0;
			putserv("%s " TOKEN_MODE " %s +o %s", bot.servnum, c->chan, cs.num);
			c->chanstamp = il;
			/* le pb ici c'est qu'on ne peut dterminer un user qui s'est fait deop UNIQUEMENT sur les
			chans regs, donc si un mec tait op sur un chan non reg, que le timestamp le fasse se deop,
			et qu'il veut reg le chan, il pourra sans etre op (bien qu'il l'tait tout de mme avant)
			Toute faon  part a on s'en fou vu que les chans sont pas reg on en a rien  battre */
			for(j = c->members;j;j = j->next) j->value.j->status &= ~(J_OP | J_VOICE);
		}
		if(c) string2scmode(&c->mode, modes, key, limit); /*burst d'un salon aprs EB -> ajout de ses modes*/
	}
	else
	{/* envoi d'un B d'un salon reg par uplink, B-back avec defmodes, ajout des deux sries de modes*/
		if((c = getchaninfo(chan)) && !IsSuspend(c) && !CJoined(c))
		{
			putserv("%s " TOKEN_BURST " %s %lu +%s %s:o", bot.servnum, c->chan, il, GetCModes(c->defmodes), cs.num);
			c->mode.modes = c->defmodes.modes;/* c'est ma connexion, donc aucun mode mmoris*/
			c->mode.limit = c->defmodes.limit;
			string2scmode(&c->mode, modes, key ? key : c->defmodes.key, limit);
			if(CLockTopic(c))
			{
				if(*c->topic) cstopic(c, c->topic);
				else if(*c->deftopic) cstopic(c, c->deftopic);
			}
			SetCJoined(c);
			c->chanstamp = il;
		}
		else if(!strcasecmp(bot.pchan, chan))
		{
#ifdef USE_WELCOMESERV
			putserv("%s " TOKEN_BURST " %s %lu +mintrs %s:ov,%s", bot.servnum, bot.pchan, il, ws.num, cs.num);
#else
			putserv("%s " TOKEN_BURST " %s %lu +mintrs %s:ov", bot.servnum, bot.pchan, il, cs.num);
#endif
			if(burst > 0) burst = -burst; /* pour remplacer le 'gotcs' local, sans rajouter de global var*/
		}
	}

	for(num = Strtok(&key, users, ',');num;num = Strtok(&key, NULL, ','))
	{
		if((flags = strchr(num, ':')))
		{
			status = 0;
			*flags = '\0';
			while (*(++flags))
			if (*flags == 'o') status |= J_OP;
			else if (*flags == 'v') status |= J_VOICE;
			else break;
		}
		if((nick = num2nickinfo(num))) add_join(nick, chan, status | J_BURST, il, c);
		else Debug(W_DESYNCH|W_WARN, "BURST %s sur %s de %s: aNick non trouv!", num, parv[1], parv[0]);
	}

	return 0;
}

/*
 * m_quit SSCCC Q :raison
 */
int m_quit(int parc, char **parv)
{
	del_nickinfo(parv[0], "QUIT");
	return 0;
}

/*
 * m_kill SSCCC D SSCC :path (raison)
 */
int m_kill(int parc, char **parv)
{
	aNick *nick;

	if(parc < 3) return Debug(W_PROTO|W_WARN, "#arg invalide: KILL de %s: arg=%d", parv[0], parc);
	if(!(nick = num2nickinfo(parv[1])) && !strncmp(parv[2], cs.nick, strlen(cs.nick)) && parv[2][strlen(cs.nick)] == ' ')
		return 0;/* GHost5*/
	else if (!nick) return Debug(W_DESYNCH|W_WARN, "KILL de %s sur %s: aNick non trouv!", parv[0], parv[1]);

	del_nickinfo(parv[1], "KILL");
	return 0;
}

/*
 * m_away SSCCC A [msg]
 */
int m_away(int parc, char **parv)
{
    aNick *nick = GetInfobyPrefix(parv[0]);

	if(!nick) return Debug(W_DESYNCH|W_WARN, "AWAY de %s: aNick non trouv!", parv[0]);
	if(parc > 1 && *parv[1]) nick->flag |= N_AWAY;
	else
	{
		nick->flag &= ~N_AWAY;
#ifdef USE_MEMOSERV
		if(nick->user) show_notes(nick);
#endif
	}
	return 0;
}

/*
 * m_whois
 */
int m_whois(int parc, char **parv)
{
	if(!strcasecmp(parv[2], cs.nick))
		putserv("%s 311 %s %s %s %s * :%s", bot.servnum, parv[0], cs.nick, cs.ident, cs.host, cs.name);
#ifdef USE_WELCOMESERV
	else if(!strcasecmp(parv[2], ws.nick))
		putserv("%s 311 %s %s %s %s * :%s", bot.servnum, parv[0], ws.nick, ws.ident, ws.host, ws.name);
#endif
	else
	{
		aNick *tmp = getnickbynick(parv[2]);
		if(!tmp) return 0;

		putserv("%s 311 %s %s %s %s * :%s", bot.servnum, parv[0], parv[2], tmp->ident, (tmp->user && tmp->flag & N_CRYPT) ? "hidden" :
#ifdef HAVE_CRYPTHOST
		tmp->crypt, tmp->name);
#else
		tmp->host, tmp->name);
#endif
		putserv("%s 312 %s %s %s :%s", bot.servnum, parv[0], parv[2], tmp->serveur->serv, tmp->name);
		if(IsOper(tmp)) putserv("%s 313 %s %s :est un IRC Oprateur tyrannique et pervers", bot.servnum, parv[0], parv[2]);
		putserv("%s 318 %s %s :Fin de la commande /WHOIS", bot.servnum, parv[0], parv[2]);
		return 0;
	}
	putserv("%s 312 %s %s %s :%s", bot.servnum, parv[0], parv[2], bot.server, bot.name);
	putserv("%s 313 %s %s :est un IRC Oprateur - Service", bot.servnum, parv[0], parv[2]);
	putserv("%s 317 %s %s %lu %lu :Secondes d'idle, secondes d'uptime", bot.servnum, parv[0], parv[2], (time(NULL) - bot.uptime), bot.uptime);
	putserv("%s 318 %s %s :Fin de la commande /WHOIS", bot.servnum, parv[0], parv[2]);
	return 0;
}

int m_topic(int parc, char **parv)
{
	aChan *chan;

	if(parc < 2) return Debug(W_PROTO|W_WARN, "#arg invalide: TOPIC de %s: arg=%d", parv[0], parc);
	if(!(chan = getchaninfo(parv[1]))) return 0;/* chan non reg*/
	if(CLockTopic(chan))
	{
		if(*chan->topic) cstopic(chan, chan->topic);
		else cstopic(chan, chan->deftopic);
	}
	else Strncpy(chan->topic, parv[NUMSERV + 1], TOPICLEN);
	return 0;
}

/*
 * m_eob SS EB SS
 */
int m_eob(int parc, char **parv)
{
	aServer *serv = num2servinfo(parv[0]);

	if (!serv || !(serv->flag & ST_BURST))
		return Debug(W_DESYNCH|W_WARN, "EOB de %s: serveur inconnu/dj synchro?", parv[0]);

	serv->flag &= ~ST_BURST;
	serv->flag |= ST_ONLINE;
	if(serv == mainhub) /* mon uplink a fini son burst, j'envoie la fin du mien!*/
	{
		aChan *chan;
		int i = 0;
		time_t current = time(NULL);

#ifdef USE_WELCOMESERV
		if(burst > 0) putserv("%s " TOKEN_BURST " %s %lu +mints %s:ov,%s", bot.servnum, bot.pchan, current, cs.num, ws.num);
#else
		if(burst > 0) putserv("%s " TOKEN_BURST " %s %lu +mints %s:ov", bot.servnum, bot.pchan, current, cs.num);
#endif
		else if(burst < 0) burst = -burst;

		for(;i < CHANHASHSIZE; i++)
		{
			for(chan = chan_tab[i];chan;chan = chan->next)
			{
				if(!CJoined(chan) && !IsSuspend(chan))
				{
					putserv("%s " TOKEN_BURST " %s %lu +%s %s:o", bot.servnum, chan->chan, current, GetCModes(chan->defmodes), cs.num);
					if(*chan->topic) cstopic(chan, chan->topic);
					else if(*chan->deftopic) cstopic(chan, chan->deftopic);
					SetCJoined(chan);
					chan->chanstamp = current;

					chan->mode.modes = chan->defmodes.modes;/* le salon est vide donc pas de modes*/
					strcpy(chan->mode.key, chan->defmodes.key);
					chan->mode.limit = chan->defmodes.limit;
				}/*end if*/
			}/* end for chan*/
		}/* end for HASH*/

		putserv("%s " TOKEN_EOB, bot.servnum);
		putserv("%s " TOKEN_EOBACK, bot.servnum);

		if(bot.dataQ)
		{
			struct timeval tv;
			float tt;

			gettimeofday(&tv, NULL);/*un peu de stats sur le burst pour *fun* */
			tt = (tv.tv_usec - burst) / 1000000.0;
			cswallops("Synchronisation de %u bytes en %12.6f s (Taux %12.6f Ko/s)",
						bot.dataQ, tt,(float) (bot.dataQ / 1024.0) / tt);
		}
		burst = 0;
	}
	alarm(ALARMINTERVAL);
	return 0;
}

int add_server(const char *name, const char *num, const char *hop, const char *proto, const char *from)
{
	aServer *links = calloc(1, sizeof(aServer));
	unsigned int servindex;

	if(!links)
		return Debug(W_MAX|W_WARN, "m_server, malloc() a chou pour le Link %s", name);

	Strncpy(links->serv, name, 150);
	Strncpy(links->num, num, NUMSERV);
	links->maxusers = base64toint(num + NUMSERV);
	links->flag = 0;
	servindex = base64toint(links->num);
	links->smask = 16;
	while(links->smask < links->maxusers) links->smask <<= 1;
	links->smask--;/* on passe de 010000  001111*/

	if(serv_tab[servindex]) return Debug(W_DESYNCH|W_WARN, "m_server: collision, le serveur %s[%s] existe dj?", name, links->num);

	links->hub = num2servinfo(from); /* ptr vers son Hub */
	serv_tab[servindex] = links;

	if(!(num_tab[servindex] = calloc(1, (links->maxusers+1) * sizeof(aNick *))))
		return Debug(W_MAX|W_WARN, "m_server, malloc() a chou pour les %d Offsets de %s", links->maxusers+1, name);

	if(atoi(hop) == 1 && strcasecmp(links->num, bot.servnum))
		mainhub = links; /*1erhub = my uplink -> je suis enregistr!*/

	if (*proto == 'J') links->flag |= ST_BURST;
	else links->flag |= ST_ONLINE;
	return 0;
}

int m_server(int parc, char **parv)
{
	if(parc < 7) return Debug(W_PROTO|W_WARN, "#arg invalide: SERVER de %s: arg=%d", parv[0], parc);

	return add_server(parv[1], parv[6], parv[2], parv[5], parv[0]);
}

static int do_squit(int server_numeric)
{
	int i = 0;

	for(;i < MAXNUM;i++)
		if(serv_tab[i] && serv_tab[i]->hub == serv_tab[server_numeric])/* i est un leaf de serveur_numeric*/
			do_squit(i);/* squit de ses leaf*/

	for(i = 0;i <= serv_tab[server_numeric]->maxusers;i++)
	{
		if(num_tab[server_numeric][i])
		{
			del_nickinfo(num_tab[server_numeric][i]->numeric, "SQUIT");
			num_tab[server_numeric][i] = NULL;
		}
	}

	free(num_tab[server_numeric]);
	num_tab[server_numeric] = NULL;
	free(serv_tab[server_numeric]);
	serv_tab[server_numeric] = NULL;
	return 1;
}

int m_squit(int parc, char **parv)
{
	int i = 0;

	if(parc < 2) return Debug(W_PROTO|W_WARN, "#arg invalide: SQUIT de %s: arg=%d", parv[0], parc);

	for(;i < MAXNUM;i++)
		if(serv_tab[i] && !strcasecmp(serv_tab[i]->serv, parv[1]))
			return do_squit(i);

	return Debug(W_DESYNCH|W_WARN, "Squit d'un serveur inconnu %s!", parv[1]);
}

int m_pass(int parc, char **parv)
{
	struct timeval tv;
	time_t current = time(NULL);

	gettimeofday(&tv, NULL);
	burst = tv.tv_usec;

	putlog(LOG_PARSES, "Connect au serveur %s", bot.ip);
	bot.lasttime = current;
	bot.lastbytes = bot.dataQ;
	if(!lastresetparse) lastresetparse = current;
	return 0;
}

int m_ping(int parc, char **parv)
{
#ifdef N2K
	struct timeval tv;
	char *theirttm = strchr(parv[3], '.');

	if(theirttm) theirttm++;
	gettimeofday(&tv, NULL);
/* AsLL */
	putserv("%s " TOKEN_PONG " %s %s %s %ld %ld.%ld", bot.servnum, bot.servnum,
			parv[0], parv[3], ((tv.tv_sec - atoi(parv[3])) * 1000 + (tv.tv_usec - atoi(theirttm)) / 1000),
			tv.tv_sec, tv.tv_usec);/* mon TS seconde.micro*/
#else
    putserv("%s " TOKEN_PONG " %s", bot.servnum, parv[1]);
#endif
    return 0;
}

static int exec_cmd(unsigned int i, unsigned int securise, aNick *nick, int parc, char **parv)
{
	char *tmp3 = NULL, buff[512];
	int len = 0;
	aChan *chaninfo = NULL;
	anAccess *acces = NULL;
#ifdef HAVE_TRACK
	struct track *track = NULL;
#endif

	if(isignore(nick) || checkflood(nick)) return 0;

#ifdef HAVE_SECURE
	if(cs_cmds[i].secure && !securise && (strcasecmp(cs_cmds[i].realcmd, "memos") || !parc || !strcasecmp(parv[1], "SEND")))
		return csntc(nick, "La commande\2 %s\2 demande des informations prives. Veuillez utiliser \2/%s %s %s\2.",
			cs_cmds[i].cmd, cs.nick, cs_cmds[i].cmd, parv2msg(parc, parv, 1, 400));
#endif

	cs_cmds[i].used++;
	if(cs_cmds[i].flag & CMD_DISABLE)
		return csntc(nick, GetReply(nick, L_CMDDISABLE));

	if(parc < cs_cmds[i].args || (!parc && ChanCmd(i)))
		return syntax_cmd(nick, i);

	if(ChanCmd(i))
	{
		if(*parv[1] != '#' || !(chaninfo = getchaninfo(parv[1])))
			return csntc(nick, GetReply(nick, L_NOSUCHCHAN), parv[1]);

		if(IsSuspend(chaninfo))
		{
			if(cs_cmds[i].level) return csntc(nick, "%s est suspendu : %s - %s", parv[1],
					chaninfo->suspend->raison, chaninfo->suspend->from);
		}

		else if(chaninfo->suspend && chaninfo->suspend->expire >= 0)
		{
			chaninfo->suspend->expire = -1;
			csjoin(chaninfo);
			putlog(LOG_CHANS, "SUSPEND OFF (Auto) %s par %s", chaninfo->chan, chaninfo->suspend->from);
		}
	}
	if(cs_cmds[i].level)
	{
		if(!nick->user || !nick->user->n)
			return csntc(nick, "%s : Vous n'tes pas logu ou vous n'avez pas encore enregistr d'Username", pasdeperm);

		if(AdmCmd(i) && nick->user->level < cs_cmds[i].level)
			return csntc(nick, GetReply(nick, L_NEEDTOBEADMIN));

		if(ChanCmd(i) && !IsAnAdmin(nick->user))
		{
			if(!(acces = GetAccessIbyUserI(nick->user, chaninfo)))
				return csntc(nick, GetReply(nick, L_YOUNOACCESSON), parv[1]);

			if(cs_cmds[i].level > acces->level)
				return csntc(nick, GetReply(nick, L_ACCESSTOOLOW), parv[1], acces->level, cs_cmds[i].level);

			if(ASuspend(acces))
				return csntc(nick, GetReply(nick, L_ACCESSSUSPENDED), parv[1]);
		}
	}

	buff[0] = '\2';
	strcpy(buff + 1, cs_cmds[i].cmd);
	len = 1 + strlen(cs_cmds[i].cmd);
	buff[len++] = '\2';
	buff[len++] = ' ';
	buff[len] = '\0';
	if(ChanCmd(i))
	{
		strcpy(buff + len, chaninfo->chan);
		len += strlen(chaninfo->chan);
		buff[len++] = ' ';
		buff[len] = '\0';
		if(parc > 1) strcpy(buff + len, (tmp3 = parv2msg(parc, parv, 2, 400)));
	}
	else
	{
		if((cs_cmds[i].secure == 1 && parc) || (parc >= 2
			&& ((!strcasecmp(cs_cmds[i].realcmd, "USER") && !strcasecmp(parv[1], "NEWPASS"))
			|| (!strcasecmp(cs_cmds[i].realcmd, "SET") && !strcasecmp(parv[1], "PASS")))))
				strcpy(buff + len, (tmp3 = parv[1]));
		else if(cs_cmds[i].secure != 2 && parc)
			strcpy(buff + len, (tmp3 = parv2msg(parc, parv, 1, 400)));
	}
	sprintf(buff + strlen(buff), nick->user ? " par %s@%s" : " par %s", nick->nick, nick->user ? nick->user->nick : "");

#ifdef HAVE_TRACK
	if(nick->user && (track = istrack(nick->user))) csntc(track->tracker, "[\2Track\2] %s", buff);
#endif
	putchan(buff);

#ifdef SQLLOG
	if(cs_cmds[i].level)
	{
		if(ChanCmd(i))
		{
			if(tmp3) tmp3[254] = 0;
			sql_query("INSERT INTO chancmdslog (user, chan, TS, cmd, log) VALUES ('%s', '%s', UNIX_TIMESTAMP(), '%s', '%s');",
				nick->user->nick, chaninfo->chan, cs_cmds[i].realcmd, tmp3);
		}
		else sql_query("INSERT INTO usercmdslog (user, TS, cmd, log) VALUES ('%s', UNIX_TIMESTAMP(), '%s', '%s');",
				nick->user->nick, cs_cmds[i].realcmd, NONE(tmp3));
	}
#else
	putlog(LOG_CMDS, "%s", buff);
#endif
	cs_cmds[i].func(nick, chaninfo, parc, parv);/*permet de passer la struct chaninfo (economie de loop)*/
	return 1;
}

/*
 * m_privmsg SSCCC P SSCCC|# :msg
 * pparv[0] = nom de la commande
 * pparv[pparc] = arguments de la commande
 * ATTENTION: pparc est le nb d'arg  partir de *parv[1]*
 */
int m_privmsg(int parc, char **parv)
{
	char *tmp, *pparv[250], *ptr = NULL;
	unsigned int pparc = 0, securise = 0;
	int i;
	aNick *nptr = GetInfobyPrefix(parv[0]);

	if(!nptr) return 0;/* la source du message n'est pas un user connu (serveur/desynch)*/

	if(*parv[1] != '#') /* Message priv */
	{
		pparv[1] = NULL;

		if((tmp = strchr(parv[1], '@'))) /* P nick@server */
		{
			*tmp = 0;
			securise = 1;
		}

		if(!*parv[2] || *parv[2] == ' ' || (strcmp(parv[1], cs.num) && strcasecmp(parv[1], cs.nick))) return 0;

		pparv[0] = (tmp = Strtok(&ptr, parv[2], ' '));
		if(!tmp) return 0;

		if((i = GetCurCmdIndexbyName(tmp)) < 0)
		{
			if(*tmp != '\1') csntc(nptr, GetReply(nptr, L_NOSUCHCMD), tmp);
			return 0;
		}

		while((tmp = Strtok(&ptr, NULL, ' '))) pparv[++pparc] = tmp;

		exec_cmd(i, securise, nptr, pparc, pparv);
	}
	else
	{  /* Message envoy sur un salon (pour chanserv)*/
		if(parv[2] && *parv[2] == bot.cara && *++parv[2] && *parv[2] != ' ')
		{
			pparv[0] = (tmp = Strtok(&ptr, parv[2], ' '));
			if(!tmp) return 0;

			if((i = GetCurCmdIndexbyName(tmp)) < 0) return 0;

			tmp = Strtok(&ptr, NULL, ' '); /* on traite le premier arg aprs la commande*/
			if(ChanCmd(i) && (!tmp || *tmp != '#')) pparv[++pparc] = parv[1]; /* si ce n'est pas un salon*/
			if(tmp) pparv[++pparc] = tmp;									/* on prend le salon o elle est tape*/

			while((tmp = Strtok(&ptr, NULL, ' '))) pparv[++pparc] = tmp;

			exec_cmd(i, securise, nptr, pparc, pparv);
		}
	}
	return 0;
}
