/* src/timers.c - fonctions de controle appelles periodiquement
 * Copyright (C) 2002-2003 Inter System (kouak@kouak.org)
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: timers.c,v 1.37 2003/12/27 16:19:27 romexzf Exp $
 */

#include "main.h"
#include "config.h"
#include "outils.h"
#include "hash.h"
#include "del_info.h"
#include "fichiers.h"
#include "cs_cmds.h"

void check_accounts(void)
{
	anUser *user, *u2 = NULL;
	int i = 0;
	time_t current = time(NULL);

	for(;i < USERHASHSIZE;i++) for(user = user_tab[i];user;user = u2)
	{
		int trouve = 1;
		u2 = user->next;
		if((user->lastseend + maxlastseen) < current && !IsAnAdmin(user) && !UNopurge(user))
			cswallops("Purge de l'username %s (Lastseen %s)", user->nick, duration((current - user->lastseend)));
		else if(UFirst(user) && (user->lastseend + REGISTER_TIMEOUT) < current)
			cswallops("Purge de l'username %s par register timeout", user->nick);
		else
		{
			if(user->n) user->lastseend = current;
			if(user->suspend && (user->suspend->debut + user->suspend->expire) < current && user->suspend->expire > 0)
				user->suspend->expire = -1;
			if(user->cantregchan > 0 && user->cantregchan < current) user->cantregchan = -1;
			trouve = 0;
		}
		if(trouve)
		{
			anAccess *access = user->accesshead, *ac = NULL;
			for(;access;access = ac)
			{
				ac = access->next;
				if(access->level == OWNERLEVEL)
				{
					char tmp[400];
					snprintf(tmp, sizeof tmp, "Salon %s desenregistr -  Purge de l'username de l'owner (Lastseen %s)",
						access->c->chan, duration(current - user->lastseend));
					del_chan(access->c, tmp);
				}
			}
			del_regnick(user);
		}
	}
	return;
}

void check_kills(void)
{
	aKill *current, *next;
	char buff[NICKLEN + 1];
	time_t ttm = time(NULL);

	if(!kill_interval) return;
	for(current = killhead;current;current = next)
	{
		int del = 1;
		next = current->next;
		if(ttm >= (current->timestamp + kill_interval))
		{
			if(!strcasecmp(current->nick->nick, current->against))
			{
				if(!current->setkill && current->why == 2)
				{
					snprintf(buff, NICKLEN, "%s%02d", CHNICKRAND, rand());
					putserv("%s " TOKEN_SVSNICK " %s :%s", bot.servnum, current->nick->numeric, buff);
				}
				else
				{
					putserv("%s " TOKEN_KILL " %s :%s (%s)", cs.num, current->nick->numeric, cs.nick,
						(current->why == 1) ? "Pseudo illgal" : "Pseudo enregistr");
					del_nickinfo(current->nick->numeric, "timer kill");
					del = 0;
				}
			}
			if(del) del_killinfo(current->nick, NULL);
		}
	}
}

int check_chan(aChan *chan)
{
	aLink *lp = NULL;
	time_t current = time(NULL);
	int etat = 0; /*1=pas d'owner, 2=pas d'access, 3=lastseend>maxlastseen*/

	if(!chan->access) etat = 2;
	else
	{
		for(lp = chan->access;lp && lp->value.a->level != OWNERLEVEL;lp = lp->next);
		if(lp && lp->value.a->lastseen > 1 && (lp->value.a->lastseen + maxlastseen) < current && !UNopurge(lp->value.a->user)) etat = 3;
		else if(!lp) etat = 1;/* pas d'owner*/
	}
	if(chan->suspend && (chan->suspend->debut + chan->suspend->expire) < current && chan->suspend->expire > 0)
	{
		chan->suspend->expire = -1;
		if(!CJoined(chan)) csjoin(chan);
		putlog(LOG_CHANS, "SUSPEND OFF (Auto) %s par %s", chan->chan, chan->suspend->from);
	}
	if(etat)
	{
		if(etat == 3 || CWantUnreg(chan))/*abandon de l'owner ou salon dj remarqu*/
		{
			char raison[300];
			if(etat == 1) snprintf(raison, sizeof raison, "Salon %s desenregistr - Aucun owner", chan->chan);
			else if(etat == 2) snprintf(raison, sizeof raison, "Salon %s desenregistr - Aucun accs", chan->chan);
			else if(etat == 3) snprintf(raison, sizeof raison, "Salon %s desenregistr - Pas de visite de l'owner depuis \2%s", chan->chan, get_time(lp->value.a->lastseen));
			del_chan(chan, raison);
			return 1;
		}
		else
		{
			if(etat == 1) cswallops("Le salon %s n'a pas d'owner. Si dans deux heure il n'en a pas retrouv un le salon sera supprim.", chan->chan);
			else if(etat == 2) cswallops("Le salon %s n'a pas d'access. Le salon va tre supprim dans deux heures si il n'a pas t trait avant.", chan->chan);
			SetCWantUnreg(chan);/* marquons le, *only* warning*/
		}
	}
	return 0;
}

void check_bans(int checkchan)/* si checkchan == 1 le salon subit les vrifications de purge*/
{
	aChan *c, *next;
	int i = 0;
	aBan *b, *b2;
	time_t current = time(NULL);

	for(;i < CHANHASHSIZE;i++)
	{
		for(c = chan_tab[i];c;c = next)
		{
			next = c->next;
			if(checkchan && check_chan(c)) continue;/* le salon vient d'tre purg*/
			if(!IsSuspend(c) && CJoined(c))
			{
				int have = 0;
				if(CFLimit(c))
				{
					int newlimit = c->users + c->limit_inc, dif = (newlimit > c->mode.limit) ? newlimit - c->mode.limit : c->mode.limit - newlimit;
					if(c->mode.limit != newlimit && dif >= c->limit_min)/* update de la limite que si*/
					{										/* a vaut le coup*/
						c->mode.limit = newlimit;
						if(!(c->mode.modes & C_MLIMIT)) c->mode.modes |= C_MLIMIT;
						putserv("%s " TOKEN_MODE " %s +l %d", cs.num, c->chan, newlimit);
					}
				}
				for(b = c->banhead;b;b = b2)
				{
					b2 = b->next;
					if(b->fin && current >= (b->debut + b->fin))
					{
						csmode(c, MODE_OBV, "-b %s", b->mask);
						del_ban(c, b);
						have = 1;
					}
				}
				if(have) write_bans(c);
			}
		}/* for(chantab)*/
	}/* for(hash)*/
	return;
}
