/* src/welcome.c - Diverses commandes sur le module welcome
 * Copyright (C) 2002-2003 Inter System (Progs@Inter-System.Net)
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: welcome.c,v 1.17 2003/12/27 16:19:27 romexzf Exp $
 */

#include "main.h"
#ifdef USE_WELCOMESERV
#include "cs_cmds.h"
#include "debug.h"
#include "config.h"
#include "outils.h"
#include "welcome.h"

struct welcomeinfo *welcomehead = NULL;
/*
 * set_welcome on/off
 */
int set_welcome(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	switch_option(nick, parc ? parv[1] : NULL, "welcome", "serveur", &ConfFlag, CF_WELCOME);
	write_welcome();
	return 1;
}

/* /z addwelcome <level> <message> */

int add_welcome(aNick *nick, aChan *c, int parc, char **parv)
{
	struct welcomeinfo *w;
	int level;

	if(!is_num(parv[1]) || (level = atoi(parv[1])) < 1 || level > 100)
		return csntc(nick, GetReply(nick, L_VALIDLEVEL));

	if((w = add_welcometolist(level, parv2msg(parc, parv, 2, 250))))
	{
		csntc(nick, "Welcome bien ajout au level %d", level);
		csntc(nick, "\2[Message de bienvenue #%d]\2 - %s", w->id, w->msg);
		write_welcome();
	}
	return 1;
}

int del_welcome(aNick *nick, aChan *c, int parc, char **parv)
{
	struct welcomeinfo *current = NULL, *previous = NULL;
	int id;

	if(parc < 1) return 0;
	if(!is_num(parv[1])) return csntc(nick, "L'id doit tre un nombre.");

	id = atoi(parv[1]);
	current = welcomehead;

	while(current)
	{
		if(id == current->id) break;
		previous = current;
		current = current->next;
	}
	if(!current)
		return csntc(nick, "Aucune message ayant pour id %d", id);

	if(!previous) welcomehead = current->next; /* ben c'etait le 1er */
	else previous->next = current->next;
	free(current);
	current = welcomehead;

	id = 1;
	while(current)
	{
		current->id = id;
		current = current->next;
		id++;
	}
	csntc(nick, "Le message ayant pour id %s a bien t supprim (un total de %d messages dsormais)", parv[1], id - 1);
	write_welcome();
	return 1;
}

int list_welcome(aNick *nick, aChan *c, int parc, char **parv)
{
	struct welcomeinfo *w;

	if(!welcomehead) return csntc(nick, "Aucun message de bienvenue.");

	csntc(nick, "\2#Id Level Message");
	for(w = welcomehead;w;w = w->next) csntc(nick, "#%d %d %s", w->id, w->level, w->msg);

	return 1;
}

void choose_welcome(const char *num)
{
	int rnd = 1;
	struct welcomeinfo *w = welcomehead;

	if(!welcomehead) return;

	while(rnd)
	{
		rnd = rand()%100;
		for(w = welcomehead;w;w = w->next) if(w->level > rnd && rnd%3) break;
		if(w) break;
		else rnd--;
	}
	putserv("%s %s %s :\2[Message de bienvenue #%d]\2 - %s\n", ws.num,
			GetConf(CF_PRIVWELCOME) ? TOKEN_PRIVMSG: TOKEN_NOTICE, num, w->id, w->msg);
	return;
}

void write_welcome()
{
	FILE *fp;
	struct welcomeinfo *w;

	if(!(fp = fopen(WELCOME_FILE, "w"))) return;
	fprintf(fp, "VERSION %d\n", USRS_VERSION);
	fprintf(fp, "%d\n%s\n%s\n", GetConf(CF_WELCOME), user_motd, admin_motd);
	for(w = welcomehead;w;w = w->next) fprintf(fp, "%d :%s\n", w->level, w->msg);
   	fclose(fp);
   	return;
}

struct welcomeinfo *add_welcometolist(int level, const char *msg)
{
	struct welcomeinfo *w, *new = malloc(sizeof(struct welcomeinfo));

   	if(!new)
   	{
	   	Debug(W_MAX, "add_loadwelcome, malloc a chou pour le welcomeinfo %s", msg);
	   	return NULL;
   	}

	for(w = welcomehead;w && w->next;w = w->next);

	if(!w) new->id = 1; /* bah c'est le 1er */
	else new->id = w->id + 1; /* on incrmente l'id */

	Strncpy(new->msg, msg, 250);
	new->next = NULL;
	new->level = level;

	if(!w) welcomehead = new; /* ben c'est le 1er */
	else w->next = new;

	return new;
}

void load_welcome(void)
{
	char buff[420], *tmp;
	int t = 0, fileversion = 1, g = 0;
	FILE *fp = fopen(WELCOME_FILE, "r");

	if(!fp) return;
	while(fgets(buff, sizeof buff, fp))
   	{
		strip_newline(buff);
		if(!strncmp(buff, "VERSION", 7))
		{
			if((tmp = strchr(buff, ' '))) tmp++;
			fileversion = atoi(tmp);
		}
		else if(t == 1 && fileversion >= 8) sscanf(buff, "%d", &g);
		else if(t == 2 && fileversion >= 8) Strncpy(user_motd, buff, 390);
		else if(t == 3 && fileversion >= 8) Strncpy(admin_motd, buff, 390);
		else
		{
			tmp = strchr(buff, ' ');
			*tmp++ = 0;
			add_welcometolist(atoi(buff), ++tmp);
		}
		t++;
	}
	fclose(fp);
	if(fileversion < 8)
	{
		if(!(fp = fopen(SETWELCOME_FILE, "r"))) return;
		if(fgets(buff, sizeof buff, fp)) sscanf(buff, "%d", &g);
		fclose(fp);
	}
	if(g) ConfFlag |= CF_WELCOME;
	return;
}
#endif
